@@ -2,7 +2,7 @@
 package File::Wildcard;
 use strict;
 
-our $VERSION = 0.05;
+our $VERSION = 0.06;
 
 =head1 NAME
 
@@ -92,6 +92,21 @@ If your original filespec started with '/' before you split it, specify
 absolute => 1. B<absolute> is not required for Windows if the path contains
 a drive specification, e.g. C:/foo/bar.
 
+=item C<case_insensitive>
+
+By default, the module will use L<Filesys::Type> to determine whether the
+file system of your wildcard is defined. This is an optional module (see
+L<Module::Optional>), and File::Wildcard will guess at case sensitivity
+based on your operating system. This will not always be correct, as the
+file system might be VFAT mounted on Linux or ODS-5 on VMS.
+
+Specifying the option C<case_insensitive> explicitly forces this behaviour
+on the wildcard.
+
+Note that File::Wildcard will use the file system of the current working
+directory if the path is not absolute. If the path is absolute, you should
+specify the case_sensitivity option explicitly.
+
 =item C<match>
 
 Optional. If you do not specify a regexp, you get all the files
@@ -146,10 +161,20 @@ inside-out gives the following: qw(a/bar/drink a/bar/ a/foo/lish a/foo/ a/).
 
 By default, globbing returns the list of files in the order in which they 
 are returned by the dirhandle (internally). If you specify sort => 1, the
-files are sorted into ASCII sequence. If you specify a CODEREF, this will
+files are sorted into ASCII sequence (case insensitively if we are operating
+that way). If you specify a CODEREF, this will
 be used as a comparison routine. Note that this takes its operands in @_,
 not in $a and $b.
 
+=item C<debug> and C<debug_output>
+
+You can enable a trace of the internal states of File::Wildcard by setting
+debug to a true value. Set debug_output to an open filehandle to get the
+trace in a file. If you are submitting bug reports for File::Wildcard, attaching
+debug trace files would be very useful.
+
+debug_output defaults to STDERR.
+
 =back
 
 =head2 match
@@ -307,11 +332,21 @@ glob(3), L<File::Find>, L<File::Find::Rule>.
 
 use Params::Validate::Dummy qw();
 use Module::Optional qw(Params::Validate :all);
+
+package Filesys::Type::Dummy;
+use strict;
+
+sub case {
+    return 'insensitive' if $^O =~ /win|dos/i;
+    return 'lower' if $^O =~ /vms/i;
+    return 'sensitive';
+}
+
+package File::Wildcard;
+use Module::Optional qw(Filesys::Type);
 use File::Spec;
 use Carp;
 
-our $case_insensitive = $^O =~ /vms|win/i;
-
 sub new {
     my $pkg = shift;
 
@@ -324,14 +359,27 @@ sub new {
           sort => { type => SCALAR | CODEREF | UNDEF, optional => 1 },
           ellipsis_order => { type => SCALAR, 
                              regex => qr/(normal|breadth-first|inside-out)/,
-                           default => 'normal' },
-          debug => 0,
+			     optional => 1,
+                            },
+	  case_insensitive => { type => SCALAR, optional =>1 },
+          debug => { type => SCALAR, optional => 1 },
+	  debug_output => 0,
         } );
 
     $par{ellipsis_order} ||= 'normal';
+    my $path = $par{path}; # $par{path} is about to be chopped up
     ($par{path},$par{absolute}) = $pkg->_split_path ( 
                                   @par{qw/path absolute follow/});
+    if (exists($par{path}) && !defined $par{case_insensitive}) {
+        my $fspath = $par{absolute} ? $path : File::Spec->curdir;
+    	my $fscase = eval {
+    		Filesys::Type::case($fspath)} ||
+		Filesys::Type::Dummy::case;
+	$par{case_insensitive} = $fscase eq 'sensitive';
+    }
 
+    $par{debug_output} ||= \*STDERR if $par{debug};
+    
     unless (exists $par{match}) {
         my $match_re = $par{absolute} ? '^/' : '^';
         for (@{$par{path}}) {
@@ -341,24 +389,32 @@ sub new {
             $match_re .= ($comp || '(.*?)') . '/';
         }
         $match_re =~ s!/$!\$!;
-        $par{match} = $case_insensitive ? qr/$match_re/i : qr/$match_re/;
+        $par{match} = $par{case_insensitive} ? qr/$match_re/i : qr/$match_re/;
     }
     
     bless \%par, $pkg;
 }
 
+sub _debug {
+    my ($self,$mess) = @_;
+
+    return unless $self->{debug};
+    my $dbug = $self->{debug_output};
+
+    print $dbug $mess;
+}
+
 sub next {
     my $self = shift;
 
     $self->_set_state( state => 'initial') unless exists $self->{state};
 
     while (!exists $self->{retval}) {
-        print STDERR "In state ".$self->{state}."\n" if $self->{debug};
+        $self->_debug("In state ".$self->{state}."\n");
         my $method = "_state_" . $self->{state};
         $self->$method;
     }
-    print STDERR "Returned ".($self->{retval} || 'undef')."\n"
-        if $self->{debug};
+    $self->_debug("Returned ".($self->{retval} || 'undef')."\n");
     my $rv = $self->{retval};
     delete $self->{retval};
 
@@ -394,7 +450,7 @@ sub reset {
     $self->_set_state( state => 'initial');
 }
 
-sub derived {
+sub _derived {
     my $self = shift;
 
     return $self->{resulting_path} unless exists $self->{derive};
@@ -489,13 +545,12 @@ sub _set_state {
 sub _push_state {
     my $self = shift;
 
-    print STDERR "Push state: ".
+    $self->_debug( "Push state: ".
            $self->{state}. " resulting_path: ".
            $self->{resulting_path}.
-           " Wildcard: " . $self->{wildcard} .
+           " Wildcard: " . ($self->{wildcard} || '') .
            " path_remaining: ".
-           join ('/',@{$self->{path_remaining}}). "\n"
-           if $self->{debug};
+           join ('/',@{$self->{path_remaining}}). "\n");
     push @{$self->{state_stack}}, { map { $_, 
           (ref($self->{$_}) eq 'ARRAY') ? [@{$self->{$_}}] : $self->{$_} }
           qw/ state path_remaining dir resulting_path / } ;
@@ -509,13 +564,12 @@ sub _pop_state {
                        ? pop ( @{$self->{state_stack}} )
                        : { state => 'finished', dir => undef };
     $self->{$_} = $newstate->{$_} for keys %$newstate;
-    print STDERR "Pop state to ".$self->{state}.
+    $self->_debug( "Pop state to ".$self->{state}.
           " resulting_path: ".
            $self->{resulting_path}.
-           " Wildcard: " . $self->{wildcard} .
+           " Wildcard: " . ($self->{wildcard} || '') .
            " path_remaining: ".
-           join ('/',@{$self->{path_remaining}}). "\n"
-           if $self->{debug};
+           join ('/',@{$self->{path_remaining}}). "\n");
 }
 
 sub _state_initial {
@@ -537,9 +591,9 @@ sub _state_nextdir {
     my $self = shift;
 
     unless (@{$self->{path_remaining}}) {
-        print STDERR "Exhaused path\n" if $self->{debug};
+        $self->_debug("Exhaused path\n");
         my $re = $self->{match};
-        $self->{retval} = $self->derived 
+        $self->{retval} = $self->_derived 
             if (-e $self->{resulting_path}) && 
             ($self->{resulting_path} =~ /$re/);
         $self->_pop_state;
@@ -547,7 +601,7 @@ sub _state_nextdir {
     }
 
     my $pathcomp = shift @{$self->{path_remaining}};
-    print STDERR "Path component '$pathcomp'\n" if $self->{debug};
+    $self->_debug("Path component '$pathcomp'\n");
     if ($pathcomp eq '') {
         my $order = $self->{ellipsis_order};
         $self->_set_state( state => 
@@ -557,6 +611,7 @@ sub _state_nextdir {
             $self->_set_state( state => 
                 ($order eq 'inside-out') ? 'ellipsis' : 'nextdir');
         }
+	
     }
     elsif ($pathcomp !~ /\?|\*/) {
         $self->{resulting_path} .= $pathcomp;
@@ -582,14 +637,14 @@ sub _state_nextdir {
         my %newstate = (
                            state => 'wildcard', 
                              dir => $wcdir, 
-                        wildcard => $case_insensitive ? 
+                        wildcard => $self->{case_insensitive} ? 
                                               qr(^$wc_re$)i : 
                                               qr(^$wc_re$));
         if ($self->{sort}) {
             my @wcmatch = grep { 
                 ($_ ne '.') && 
                 ($_ ne '..') && 
-                ($case_insensitive ? /$wc_re/i : /$wc_re/)}
+                ($self->{case_insensitive} ? /$wc_re/i : /$wc_re/)}
                 readdir($wcdir);
 
             if ($^O =~ /vms/i) {
@@ -598,6 +653,8 @@ sub _state_nextdir {
             
             @wcmatch = (ref($self->{sort}) eq 'CODE') ?
                        (sort {&{$self->{sort}}($a,$b)} @wcmatch) :
+		       $self->{case_insensitive} ?
+		       (sort {lc($a) cmp lc($b)} @wcmatch) :
                        (sort @wcmatch);
             $newstate{state} = 'wildcard_sorted';
             $newstate{dir} = sub {shift @wcmatch};
